@@ -8,7 +8,7 @@ use AI::MXNet::Function::Parameters;
 
 =head1 NAME
 
-AI::MXNet::Executor::Group - Manager for a group of executors working in different contexts.
+    AI::MXNet::Executor::Group - Manager for a group of executors working in different contexts.
 =cut
 
 func _split_input_slice($batch_size, $work_load_list)
@@ -299,11 +299,11 @@ sub BUILD
 
 =decide_slices
 
-Decide the slices for each context according to the workload.
+    Decide the slices for each context according to the workload.
 
-Parameters
-----------
-$data_shapes : ArrayRef[AI::MXNet::DataDesc]
+    Parameters
+    ----------
+    $data_shapes : ArrayRef[AI::MXNet::DataDesc]
 =cut
 
 method decide_slices(ArrayRef[AI::MXNet::DataDesc] $data_shapes)
@@ -433,16 +433,16 @@ method _collect_arrays()
     }
 }
 
-=method bind_exec
+=head2 bind_exec
 
-Bind executors on their respective devices.
+    Bind executors on their respective devices.
 
-Parameters
-----------
-$data_shapes  : ArrayRef[AI::MXNet::DataDesc]
-$label_shapes : Maybe[ArrayRef[AI::MXNet::DataDesc]]
-$shared_group : Maybe[AI::MXNet::DataParallelExecutorGroup]
-$reshape      : Bool
+    Parameters
+    ----------
+    $data_shapes  : ArrayRef[AI::MXNet::DataDesc]
+    $label_shapes : Maybe[ArrayRef[AI::MXNet::DataDesc]]
+    $shared_group : Maybe[AI::MXNet::DataParallelExecutorGroup]
+    $reshape      : Bool
 =cut
 
 method bind_exec(
@@ -491,12 +491,12 @@ method bind_exec(
 
 =head2 reshape
 
-Reshape executors.
+    Reshape executors.
 
-Parameters
-----------
-$data_shapes : ArrayRef[AI::MXNet::DataDesc]
-$label_shapes : Maybe[ArrayRef[AI::MXNet::DataDesc]]
+    Parameters
+    ----------
+    $data_shapes : ArrayRef[AI::MXNet::DataDesc]
+    $label_shapes : Maybe[ArrayRef[AI::MXNet::DataDesc]]
 =cut
 
 
@@ -513,16 +513,16 @@ method reshape(
     $self->bind_exec($data_shapes, $label_shapes, undef, 1);
 }
 
-=head set_params
+=head2 set_params
 
-Assign, i.e. copy parameters to all the executors.
+    Assign, i.e. copy parameters to all the executors.
 
-Parameters
-----------
-$arg_params : HashRef[AI::MXNet::NDArray]
-    A dictionary of name to AI::MXNet::NDArray parameter mapping.
-$aux_params : HashRef[AI::MXNet::NDArray]
-    A dictionary of name to AI::MXNet::NDArray auxiliary variable mapping.
+    Parameters
+    ----------
+    $arg_params : HashRef[AI::MXNet::NDArray]
+        A dictionary of name to AI::MXNet::NDArray parameter mapping.
+    $aux_params : HashRef[AI::MXNet::NDArray]
+        A dictionary of name to AI::MXNet::NDArray auxiliary variable mapping.
 =cut
 
 method set_params(HashRef[AI::MXNet::NDArray] $arg_params, HashRef[AI::MXNet::NDArray] $aux_params)
@@ -532,18 +532,18 @@ method set_params(HashRef[AI::MXNet::NDArray] $arg_params, HashRef[AI::MXNet::ND
 
 =head2 get_params
 
-Copy data from each executor to arg_params and aux_params.
+    Copy data from each executor to arg_params and aux_params.
 
-Parameters
-----------
-$arg_params : HashRef[AI::MXNet::NDArray]
-    target parameter arrays
-$aux_params : HashRef[AI::MXNet::NDArray]
-    target aux arrays
+    Parameters
+    ----------
+    $arg_params : HashRef[AI::MXNet::NDArray]
+        target parameter arrays
+    $aux_params : HashRef[AI::MXNet::NDArray]
+        target aux arrays
 
-Notes
------
-- This function will inplace update the NDArrays in arg_params and aux_params.
+    Notes
+    -----
+    - This function will inplace update the NDArrays in arg_params and aux_params.
 =cut
 
 method get_params(HashRef[AI::MXNet::NDArray] $arg_params, HashRef[AI::MXNet::NDArray] $aux_params)
@@ -592,16 +592,16 @@ method set_states($states, $value)
 
 =head2 forward
 
-Split the data_batch according to a workload and run forward on each devices.
+    Split the data_batch according to a workload and run forward on each devices.
 
-Parameters
-----------
-data_batch : AI::MXNet::DataBatch
-Or could be any object implementing similar interface.
+    Parameters
+    ----------
+    data_batch : AI::MXNet::DataBatch
+    Or could be any object implementing similar interface.
 
-is_train : bool
-The hint for the backend, indicating whether we are during training phase.
-Default is undef, then the value $self->for_training will be used.
+    is_train : bool
+    The hint for the backend, indicating whether we are during training phase.
+    Default is undef, then the value $self->for_training will be used.
 =cut
 
 
@@ -641,21 +641,21 @@ method get_output_shapes()
 
 =head2 get_outputs
 
-Gets outputs of the previous forward computation.
+    Gets outputs of the previous forward computation.
 
-Parameters
-----------
-merge_multi_context : bool
-Default is 1. In the case when data-parallelism is used, the outputs
-will be collected from multiple devices. A 1 value indicates that we
-should merge the collected results so that they look like from a single
-executor.
+    Parameters
+    ----------
+    merge_multi_context : bool
+    Default is 1. In the case when data-parallelism is used, the outputs
+    will be collected from multiple devices. A 1 value indicates that we
+    should merge the collected results so that they look like from a single
+    executor.
 
-Returns
--------
-If merge_multi_context is 1, it is [$out1, $out2]. Otherwise, it
-is [[$out1_dev1, $out1_dev2], [$out2_dev1, $out2_dev2]]. All the output
-elements are `AI::MXNet::NDArray`.
+    Returns
+    -------
+    If merge_multi_context is 1, it is [$out1, $out2]. Otherwise, it
+    is [[$out1_dev1, $out1_dev2], [$out2_dev1, $out2_dev2]]. All the output
+    elements are `AI::MXNet::NDArray`.
 =cut
 
 method get_outputs(Bool $merge_multi_context=1)
@@ -679,21 +679,21 @@ method get_outputs(Bool $merge_multi_context=1)
 
 =head2  get_input_grads
 
-Get the gradients with respect to the inputs of the module.
+    Get the gradients with respect to the inputs of the module.
 
-Parameters
-----------
-merge_multi_context : bool
-Default is 1. In the case when data-parallelism is used, the outputs
-will be collected from multiple devices. A 1 value indicates that we
-should merge the collected results so that they look like from a single
-executor.
+    Parameters
+    ----------
+    merge_multi_context : bool
+    Default is 1. In the case when data-parallelism is used, the outputs
+    will be collected from multiple devices. A 1 value indicates that we
+    should merge the collected results so that they look like from a single
+    executor.
 
-Returns
--------
-If merge_multi_context is 1, it is [$grad1, $grad2]. Otherwise, it
-is [[$grad1_dev1, $grad1_dev2], [$grad2_dev1, $grad2_dev2]]. All the output
-elements are AI::MXNet::NDArray.
+    Returns
+    -------
+    If merge_multi_context is 1, it is [$grad1, $grad2]. Otherwise, it
+    is [[$grad1_dev1, $grad1_dev2], [$grad2_dev1, $grad2_dev2]]. All the output
+    elements are AI::MXNet::NDArray.
 =cut
 
 method get_input_grads(Bool $merge_multi_context=1)
@@ -708,16 +708,16 @@ method get_input_grads(Bool $merge_multi_context=1)
 
 =head2 backward
 
-Run backward on all devices. A backward should be called after
-a call to the forward function. Backward cannot be called unless
-$self->for_training is 1.
+    Run backward on all devices. A backward should be called after
+    a call to the forward function. Backward cannot be called unless
+    $self->for_training is 1.
 
-Parameters
-----------
-out_grads : NDArray or array ref of NDArray, optional
-Gradient on the outputs to be propagated back.
-This parameter is only needed when bind is called
-on outputs that are not a loss function.
+    Parameters
+    ----------
+    out_grads : NDArray or array ref of NDArray, optional
+    Gradient on the outputs to be propagated back.
+    This parameter is only needed when bind is called
+    on outputs that are not a loss function.
 =cut
 
 method backward(Maybe[AI::MXNet::NDArray|ArrayRef[AI::MXNet::NDArray]] $out_grads=)
@@ -749,14 +749,14 @@ method backward(Maybe[AI::MXNet::NDArray|ArrayRef[AI::MXNet::NDArray]] $out_grad
 
 =head2 update_metric
 
-Accumulate the performance according to eval_metric on all devices.
+    Accumulate the performance according to eval_metric on all devices.
 
-Parameters
-----------
-eval_metric : AI::MXNet::EvalMetric
-    The metric used for evaluation.
-labels : array ref of NDArray
-    Typically comes from label of AI::MXNet::DataBatch.
+    Parameters
+    ----------
+    eval_metric : AI::MXNet::EvalMetric
+        The metric used for evaluation.
+    labels : array ref of NDArray
+        Typically comes from label of AI::MXNet::DataBatch.
 =cut
 
 method update_metric(AI::MXNet::EvalMetric $eval_metric, ArrayRef[AI::MXNet::NDArray] $labels)
@@ -957,14 +957,14 @@ method _bind_ith_exec(
 
 =head2 _sliced_shape
 
-Get the sliced shapes for the i-th executor.
+    Get the sliced shapes for the i-th executor.
 
-Parameters
-----------
-shapes : array ref of (str, array ref)
-    The original (name, shape) pairs.
-i : int
-Which executor we are dealing with.
+    Parameters
+    ----------
+    shapes : array ref of (str, array ref)
+        The original (name, shape) pairs.
+    i : int
+    Which executor we are dealing with.
 =cut
 
 method _sliced_shape(ArrayRef[AI::MXNet::DataDesc] $shapes, Int $i, ArrayRef[Int] $major_axis)
@@ -989,11 +989,11 @@ method _sliced_shape(ArrayRef[AI::MXNet::DataDesc] $shapes, Int $i, ArrayRef[Int
 
 =head2 install_monitor
 
-Install monitor on all executors
+    Install monitor on all executors
 
-Parameters
-----------
-$mon : AI::MXNet::Monitor
+    Parameters
+    ----------
+    $mon : AI::MXNet::Monitor
 =cut
 
 method install_monitor(AI::MXNet::Monitor $mon)
